==> gdal-2.2.3-4-src.tar.bz2: osgeo4w/package.sh <==
#!/bin/bash

# http://download.osgeo.org/gdal/2.2.2/gdal-2.2.2.tar.gz

# Other Sources:
# https://raw.githubusercontent.com/Esri/file-geodatabase-api/master/FileGDB_API_1.5/FileGDB_API_1_5_VS2015.zip
# http://downloads.hexagongeospatial.com/software/2016/ECWJP2SDK/erdas-ecw-sdk-5.3.0-win.zip
# http://bin.lizardtech.com/download/developer/MrSID_DSDK-9.5.4.4703-win32-vc14.zip
# http://bin.lizardtech.com/download/developer/MrSID_DSDK-9.5.4.4703-win64-vc14.zip
# http://go.microsoft.com/fwlink/?LinkID=239647&clcid=0x409 => sqlncli-x86.msi
# http://go.microsoft.com/fwlink/?LinkID=239648&clcid=0x409 => sqlncli-x64.msi
# msiexec /i sqlncli.msi ADDLOCAL=ALL APPGUID={0CC618CE-F36A-415E-84B4-FB1BFF6967E1} (with SDK)

set -e

P=gdal
V=2.2.3
B=4


#
# run batch file and source environment changes
#
fetchenv() {
	local IFS
	IFS=
	batch=$1	
	shift
	srcenv=$(mktemp /tmp/srcenv.XXXXXXXXXX)
	dstenv=$(mktemp /tmp/dstenv.XXXXXXXXXX)
	diffenv=$(mktemp /tmp/diffenv.XXXXXXXXXX)
	cmd /c set >$srcenv
	cmd /c call `cygpath -w $batch` "$@" \>nul 2\>nul \& set >$dstenv
	/usr/bin/diff -u $srcenv $dstenv | /usr/bin/sed -f $D/envdiff.sed >$diffenv
	. $diffenv
	PATH=$PATH:/bin:/usr/bin:/cygdrive/c/windows32/system32:/cygdrive/c/windows
	echo PATH:$PATH PYTHONHOME:$PYTHONPATH PYTHONPATH:$PYTHONPATH
	rm -f $srcenv $dstenv $diffenv
}

export ARCH=$1
case "$ARCH" in
x86)
	export OSGEO4W_ROOT="C:\\OSGeo4W"
	BITS=32
	VCARCH=x86
	ECWARCH=win32
	FGDBARCH=
	HINT=gdal32.hint
	;;

x86_64)
	export OSGEO4W_ROOT="C:\\OSGeo4W64"
	BITS=64
	VCARCH=x64
	ECWARCH=x64
	FGDBARCH=64
	HINT=gdal.hint
	;;

*)
	echo "Unsupported arch $ARCH" >&2
	exit 1
	;;
esac

for i in $(seq 9 -1 2); do
	[ -f package-$ARCH.$(( i - 1 )).log ] && mv package-$ARCH.$(( i - 1 )).log package-$ARCH.$i.log 
done
[ -f package-$ARCH.log ] && mv package-$ARCH.log package-$ARCH.1.log 

[ -d log ] || mkdir log
if tty -s; then
	exec > >(tee log/package-$ARCH.log) 2>&1
else
	exec >log/package.log 2>&1
fi

export D=$PWD
fetchenv $OSGEO4W_ROOT/bin/o4w_env.bat

export P V B ARCH

export EXT_NMAKE_OPT=$PWD/nmake$BITS.opt
export FGDB_SDK=$PWD/../../filegdb
export ECW_SDK=$PWD/../../ecw-5
export MRSID_SDK=$PWD/../../MrSID_DSDK-9.5.4.4703-win$BITS-vc14

R=../../../$ARCH/release/$P
for i in python oracle filegdb ecw mrsid sosi mss; do
	[ -d $R/$P-$i ] || mkdir -p $R/$P-$i
done
[ -d $R/python3-$P ] || mkdir -p $R/python3-$P

export DESTDIR=$PWD/install$BITS
export PY2DESTDIR=$PWD/pyinstall2-$BITS
export PY3DESTDIR=$PWD/pyinstall3-$BITS

set -x

rm -rf $DESTDIR
rm -rf $PY2DESTDIR
rm -rf $PY3DESTDIR

mkdir -p $DESTDIR/etc/ini
mkdir -p $DESTDIR/share/gdal

fetchenv /cygdrive/c/Program\ Files*/Microsoft\ Visual\ Studio\ 14.0/VC/vcvarsall.bat $VCARCH
PATH=/cygdrive/c/Program\ Files*/Microsoft\ Visual\ Studio\ 14.0/Common7/IDE:$PATH

cp ini.bat $DESTDIR/etc/ini/$P.bat

mkdir -p $PY2DESTDIR/etc/postinstall
cp $P-python.postinstall $PY2DESTDIR/etc/postinstall/$P-python.bat

cp $HINT $R/setup.hint
cp $P-python.hint $R/$P-python/setup.hint
cp python3-gdal.hint $R/python3-$P/setup.hint
cp $P-oracle.hint $R/$P-oracle/setup.hint
cp $P-filegdb.hint $R/$P-filegdb/setup.hint
cp $P-ecw.hint $R/$P-ecw/setup.hint
cp $P-mrsid.hint $R/$P-mrsid/setup.hint
cp $P-sosi.hint $R/$P-sosi/setup.hint
cp $P-mss.hint $R/$P-mss/setup.hint

cp ../LICENSE.TXT $R/$P-$V-$B.txt
cp ../LICENSE.TXT $R/$P-oracle/$P-oracle-$V-$B.txt
cp ../LICENSE.TXT $R/$P-mss/$P-mss-$V-$B.txt
cp ../LICENSE.TXT $R/$P-python/$P-python-$V-$B.txt
cp ../LICENSE.TXT $R/python3-$P/python3-$P-$V-$B.txt
cp $FGDB_SDK/license/userestrictions.txt $R/$P-filegdb/$P-filegdb-$V-$B.txt
cp $ECW_SDK/EULA.rtf $R/$P-ecw/$P-ecw-$V-$B.rtf
cp $MRSID_SDK/LICENSE.pdf $R/$P-mrsid/$P-mrsid-$V-$B.pdf

cd ..

[ -f osgeo4w/patched-nas-update-attributes ] || patch -p1 <osgeo4w/nas-update-attributes.diff >osgeo4w/patched-nas-update-attributes

for i in clean default install devinstall; do
	[ -f osgeo4w/no$i ] && continue

	nmake /f makefile.vc \
		EXT_NMAKE_OPT=$(cygpath -w $EXT_NMAKE_OPT) \
		GDAL_HOME=$(cygpath -w $DESTDIR) \
		FGDB_SDK=$(cygpath -w $FGDB_SDK) \
		ECWDIR=$(cygpath -w $ECW_SDK) \
		MRSID_SDK=$(cygpath -w $MRSID_SDK) \
		$i
done

[ -f osgeo4w/nopackage ] && exit

cd swig

(
	set >../osgeo4w/log/python2.env

	for i in clean python; do
		[ -f ../osgeo4w/no$i ] && continue
		nmake /f makefile.vc EXT_NMAKE_OPT=$(cygpath -w $EXT_NMAKE_OPT) $i
	done
	cd python
	mkdir -p $PY2DESTDIR/apps/$P/python
	python setup.py install --prefix=$(cygpath -w $PY2DESTDIR/apps/python27) --single-version-externally-managed --record ..\\..\\osgeo4w\\log\\record-2-$ARCH.log
)

(
	fetchenv $OSGEO4W_ROOT/bin/py3_env.bat
	type python
	set >../osgeo4w/log/python3.env

	for i in clean python; do
		[ -f ../osgeo4w/no$i ] && continue
		nmake /f makefile.vc PYDIR=$(cygpath -w $OSGEO4W_ROOT/apps/Python36) EXT_NMAKE_OPT=$(cygpath -w $EXT_NMAKE_OPT) $i
	done

	cd python
	mkdir -p $PY3DESTDIR/apps/$P/Python36
	python setup.py install --prefix=$(cygpath -w $PY3DESTDIR/apps/Python36) --single-version-externally-managed --record ..\\..\\osgeo4w\\log\\record-3-$ARCH.log
)

cd python
mkdir -p $PY2DESTDIR/bin
cp scripts/*.py $PY2DESTDIR/bin
mkdir -p $PY3DESTDIR/apps/Python36/Scripts
cp scripts/*.py $PY3DESTDIR/apps/Python36/Scripts

cd ../../osgeo4w

cp $FGDB_SDK/bin$FGDBARCH/FileGDBAPI.dll $DESTDIR/bin
cp $ECW_SDK/bin/vc140/$ECWARCH/NCSEcw.dll $DESTDIR/bin
cp $MRSID_SDK/Raster_DSDK/lib/lti_dsdk_cdll_9.5.dll $DESTDIR/bin
cp $MRSID_SDK/Raster_DSDK/lib/tbb.dll $DESTDIR/bin
cp $MRSID_SDK/Raster_DSDK/lib/lti_dsdk_9.5.dll $DESTDIR/bin
cp $MRSID_SDK/Lidar_DSDK/lib/lti_lidar_dsdk_1.1.dll $DESTDIR/bin

tar -C pyinstall2-$BITS --exclude="*.pyc" -cjvf $R/$P-python/$P-python-$V-$B.tar.bz2 \
	apps/Python27 bin etc
tar -C pyinstall3-$BITS --exclude="*.pyc" --exclude __pycache__ -cjvf $R/python3-$P/python3-$P-$V-$B.tar.bz2 \
	apps/Python36
tar -C install$BITS --remove-files -cjvf $R/$P-filegdb/$P-filegdb-$V-$B.tar.bz2 \
	bin/gdalplugins/ogr_FileGDB.dll \
	bin/FileGDBAPI.dll
tar -C install$BITS --remove-files -cjvf $R/$P-sosi/$P-sosi-$V-$B.tar.bz2 \
	bin/gdalplugins/ogr_SOSI.dll
tar -C install$BITS --remove-files -cjvf $R/$P-oracle/$P-oracle-$V-$B.tar.bz2 \
	bin/gdalplugins/gdal_GEOR.dll \
	bin/gdalplugins/ogr_OCI.dll
tar -C install$BITS --remove-files -cjvf $R/$P-mss/$P-mss-$V-$B.tar.bz2 \
	bin/gdalplugins/ogr_MSSQLSpatial.dll
tar -C install$BITS --remove-files -cjvf $R/$P-ecw/$P-ecw-$V-$B.tar.bz2 \
	bin/gdalplugins/gdal_ECW_JP2ECW.dll \
	bin/NCSEcw.dll
tar -C install$BITS --remove-files -cjvf $R/$P-mrsid/$P-mrsid-$V-$B.tar.bz2 \
	bin/gdalplugins/gdal_MG4Lidar.dll \
	bin/gdalplugins/gdal_MrSID.dll \
	bin/lti_dsdk_cdll_9.5.dll \
	bin/lti_dsdk_9.5.dll \
	bin/lti_lidar_dsdk_1.1.dll \
	bin/tbb.dll
tar -C install$BITS --remove-files -cjvf $R/$P-$V-$B.tar.bz2 \
	bin \
	etc \
	include \
	lib \
	share
tar -C .. -cjvf $R/$P-$V-$B-src.tar.bz2 \
	osgeo4w/$HINT \
	osgeo4w/gdal-ecw.hint \
	osgeo4w/gdal-filegdb.hint \
	osgeo4w/gdal-mrsid.hint \
	osgeo4w/gdal-oracle.hint \
	osgeo4w/gdal-python.hint \
	osgeo4w/python3-gdal.hint \
	osgeo4w/gdal-sosi.hint \
	osgeo4w/gdal-mss.hint \
	osgeo4w/gdal-python.postinstall \
	osgeo4w/ini.bat \
	osgeo4w/nas-update-attributes.diff \
	osgeo4w/nmake$BITS.opt \
	osgeo4w/package.sh \
	osgeo4w/envdiff.sed

==> gdal-2.2.4-1-src.tar.bz2: osgeo4w/package.sh <==
#!/bin/bash

# http://download.osgeo.org/gdal/2.2.4/gdal-2.2.4.tar.gz

# Other Sources:
# https://raw.githubusercontent.com/Esri/file-geodatabase-api/master/FileGDB_API_1.5/FileGDB_API_1_5_VS2015.zip
# http://downloads.hexagongeospatial.com/software/2016/ECWJP2SDK/erdas-ecw-sdk-5.3.0-win.zip
# http://bin.lizardtech.com/download/developer/MrSID_DSDK-9.5.4.4703-win32-vc14.zip
# http://bin.lizardtech.com/download/developer/MrSID_DSDK-9.5.4.4703-win64-vc14.zip
# http://go.microsoft.com/fwlink/?LinkID=239647&clcid=0x409 => sqlncli-x86.msi
# http://go.microsoft.com/fwlink/?LinkID=239648&clcid=0x409 => sqlncli-x64.msi
# msiexec /i sqlncli.msi ADDLOCAL=ALL APPGUID={0CC618CE-F36A-415E-84B4-FB1BFF6967E1} (with SDK)

set -e

P=gdal
V=2.2.4
B=1

# override msys
cat() { /bin/cat "$@"; }
cp() { /bin/cp "$@"; }
rm() { /bin/rm "$@"; }
mkdir() { /bin/mkdir "$@"; }
tar() { /bin/tar "$@"; }
ls() { /bin/ls "$@"; }
sed() { /bin/sed "$@"; }
diff() { /bin/diff "$@"; }
declare -x cat cp rm mkdir tar ls sed diff

#
# run batch file and source environment changes
#
fetchenv() {
	local IFS
	IFS=
	batch=$1	
	shift
	srcenv=$(mktemp /tmp/srcenv.XXXXXXXXXX)
	dstenv=$(mktemp /tmp/dstenv.XXXXXXXXXX)
	diffenv=$(mktemp /tmp/diffenv.XXXXXXXXXX)
	cmd /c set >$srcenv
	cmd /c call `cygpath -w $batch` "$@" \>nul 2\>nul \& set >$dstenv
	diff -u $srcenv $dstenv | sed -f $D/envdiff.sed >$diffenv
	. $diffenv
	PATH=$PATH:/bin:/usr/bin:/cygdrive/c/windows32/system32:/cygdrive/c/windows
	rm -f $srcenv $dstenv $diffenv
}

export ARCH=$1
case "$ARCH" in
x86)
	export OSGEO4W_ROOT="C:\\OSGeo4W"
	BITS=32
	VCARCH=x86
	ECWARCH=win32
	FGDBARCH=
	HINT=gdal32.hint
	;;

x86_64)
	export OSGEO4W_ROOT="C:\\OSGeo4W64"
	BITS=64
	VCARCH=x64
	ECWARCH=x64
	FGDBARCH=64
	HINT=gdal.hint
	;;

*)
	echo "Unsupported arch $ARCH" >&2
	exit 1
	;;
esac

for i in $(seq 9 -1 2); do
	[ -f package-$ARCH.$(( i - 1 )).log ] && mv package-$ARCH.$(( i - 1 )).log package-$ARCH.$i.log 
done
[ -f package-$ARCH.log ] && mv package-$ARCH.log package-$ARCH.1.log 

[ -d log ] || mkdir log
if tty -s; then
	exec > >(tee log/package-$ARCH.log) 2>&1
else
	exec >log/package.log 2>&1
fi

export D=$PWD
fetchenv $OSGEO4W_ROOT/bin/o4w_env.bat

export P V B ARCH

export EXT_NMAKE_OPT=$PWD/nmake$BITS.opt
export FGDB_SDK=$PWD/../../filegdb
export ECW_SDK=$PWD/../../ecw-5
export MRSID_SDK=$PWD/../../MrSID_DSDK-9.5.4.4703-win$BITS-vc14

R=../../../$ARCH/release/$P
for i in python oracle filegdb ecw mrsid sosi mss; do
	[ -d $R/$P-$i ] || mkdir -p $R/$P-$i
done
[ -d $R/python3-$P ] || mkdir -p $R/python3-$P

export DESTDIR=$PWD/install$BITS
export PY2DESTDIR=$PWD/pyinstall2-$BITS
export PY3DESTDIR=$PWD/pyinstall3-$BITS

rm -rf $DESTDIR $PY2DESTDIR $PY3DESTDIR

mkdir -p $DESTDIR/etc/ini
mkdir -p $DESTDIR/share/gdal

fetchenv /cygdrive/c/Program\ Files*/Microsoft\ Visual\ Studio\ 14.0/VC/vcvarsall.bat $VCARCH
PATH=/cygdrive/c/Program\ Files*/Microsoft\ Visual\ Studio\ 14.0/Common7/IDE:$PATH

cp ini.bat $DESTDIR/etc/ini/$P.bat

mkdir -p $PY2DESTDIR/etc/postinstall
cp $P-python.postinstall $PY2DESTDIR/etc/postinstall/$P-python.bat

cp $HINT $R/setup.hint
cp $P-python.hint $R/$P-python/setup.hint
cp python3-gdal.hint $R/python3-$P/setup.hint
cp $P-oracle.hint $R/$P-oracle/setup.hint
cp $P-filegdb.hint $R/$P-filegdb/setup.hint
cp $P-ecw.hint $R/$P-ecw/setup.hint
cp $P-mrsid.hint $R/$P-mrsid/setup.hint
cp $P-sosi.hint $R/$P-sosi/setup.hint
cp $P-mss.hint $R/$P-mss/setup.hint

cp ../LICENSE.TXT $R/$P-$V-$B.txt
cp ../LICENSE.TXT $R/$P-oracle/$P-oracle-$V-$B.txt
cp ../LICENSE.TXT $R/$P-mss/$P-mss-$V-$B.txt
cp ../LICENSE.TXT $R/$P-python/$P-python-$V-$B.txt
cp ../LICENSE.TXT $R/python3-$P/python3-$P-$V-$B.txt
cp $FGDB_SDK/license/userestrictions.txt $R/$P-filegdb/$P-filegdb-$V-$B.txt
cp $ECW_SDK/EULA.rtf $R/$P-ecw/$P-ecw-$V-$B.rtf
cp $MRSID_SDK/LICENSE.pdf $R/$P-mrsid/$P-mrsid-$V-$B.pdf

cd ..

for i in clean default install devinstall; do
	[ -f osgeo4w/no$i ] && continue

	nmake /f makefile.vc \
		EXT_NMAKE_OPT=$(cygpath -w $EXT_NMAKE_OPT) \
		GDAL_HOME=$(cygpath -w $DESTDIR) \
		FGDB_SDK=$(cygpath -w $FGDB_SDK) \
		ECWDIR=$(cygpath -w $ECW_SDK) \
		MRSID_SDK=$(cygpath -w $MRSID_SDK) \
		$i
done

[ -f osgeo4w/nopackage ] && exit

cd swig

(
	export DISTUTILS_USE_SDK=1 MSSdk=1

	nmake /f makefile.vc EXT_NMAKE_OPT=$(cygpath -w $EXT_NMAKE_OPT) clean python

	cd python
	mkdir -p $PY2DESTDIR/apps/$P/python
	python setup.py install --prefix=$(cygpath -w $PY2DESTDIR/apps/python27) --single-version-externally-managed --record ..\\..\\osgeo4w\\log\\record-2-$ARCH.log
)

(
	fetchenv $OSGEO4W_ROOT/bin/py3_env.bat

	nmake /f makefile.vc PYDIR=$(cygpath -w $OSGEO4W_ROOT/apps/Python36) EXT_NMAKE_OPT=$(cygpath -w $EXT_NMAKE_OPT) clean python

	cd python
	mkdir -p $PY3DESTDIR/apps/$P/Python36
	python setup.py install --prefix=$(cygpath -w $PY3DESTDIR/apps/Python36) --single-version-externally-managed --record ..\\..\\osgeo4w\\log\\record-3-$ARCH.log
)

cd python
mkdir -p $PY2DESTDIR/bin
cp scripts/*.py $PY2DESTDIR/bin
mkdir -p $PY3DESTDIR/apps/Python36/Scripts
cp scripts/*.py $PY3DESTDIR/apps/Python36/Scripts

cd ../../osgeo4w

cp $FGDB_SDK/bin$FGDBARCH/FileGDBAPI.dll $DESTDIR/bin
cp $ECW_SDK/bin/vc140/$ECWARCH/NCSEcw.dll $DESTDIR/bin
cp $MRSID_SDK/Raster_DSDK/lib/lti_dsdk_cdll_9.5.dll $DESTDIR/bin
cp $MRSID_SDK/Raster_DSDK/lib/tbb.dll $DESTDIR/bin
cp $MRSID_SDK/Raster_DSDK/lib/lti_dsdk_9.5.dll $DESTDIR/bin
cp $MRSID_SDK/Lidar_DSDK/lib/lti_lidar_dsdk_1.1.dll $DESTDIR/bin

tar -C $PY2DESTDIR --exclude="*.pyc" -cjvf $R/$P-python/$P-python-$V-$B.tar.bz2 \
	apps/Python27 bin etc
tar -C $PY3DESTDIR --exclude="*.pyc" --exclude __pycache__ -cjvf $R/python3-$P/python3-$P-$V-$B.tar.bz2 \
	apps/Python36
tar -C install$BITS --remove-files -cjvf $R/$P-filegdb/$P-filegdb-$V-$B.tar.bz2 \
	bin/gdalplugins/ogr_FileGDB.dll \
	bin/FileGDBAPI.dll
tar -C install$BITS --remove-files -cjvf $R/$P-sosi/$P-sosi-$V-$B.tar.bz2 \
	bin/gdalplugins/ogr_SOSI.dll
tar -C install$BITS --remove-files -cjvf $R/$P-oracle/$P-oracle-$V-$B.tar.bz2 \
	bin/gdalplugins/gdal_GEOR.dll \
	bin/gdalplugins/ogr_OCI.dll
tar -C install$BITS --remove-files -cjvf $R/$P-mss/$P-mss-$V-$B.tar.bz2 \
	bin/gdalplugins/ogr_MSSQLSpatial.dll
tar -C install$BITS --remove-files -cjvf $R/$P-ecw/$P-ecw-$V-$B.tar.bz2 \
	bin/gdalplugins/gdal_ECW_JP2ECW.dll \
	bin/NCSEcw.dll
tar -C install$BITS --remove-files -cjvf $R/$P-mrsid/$P-mrsid-$V-$B.tar.bz2 \
	bin/gdalplugins/gdal_MG4Lidar.dll \
	bin/gdalplugins/gdal_MrSID.dll \
	bin/lti_dsdk_cdll_9.5.dll \
	bin/lti_dsdk_9.5.dll \
	bin/lti_lidar_dsdk_1.1.dll \
	bin/tbb.dll
tar -C install$BITS --remove-files -cjvf $R/$P-$V-$B.tar.bz2 \
	bin \
	etc \
	include \
	lib \
	share
tar -C .. -cjvf $R/$P-$V-$B-src.tar.bz2 \
	osgeo4w/$HINT \
	osgeo4w/gdal-ecw.hint \
	osgeo4w/gdal-filegdb.hint \
	osgeo4w/gdal-mrsid.hint \
	osgeo4w/gdal-oracle.hint \
	osgeo4w/gdal-python.hint \
	osgeo4w/python3-gdal.hint \
	osgeo4w/gdal-sosi.hint \
	osgeo4w/gdal-mss.hint \
	osgeo4w/gdal-python.postinstall \
	osgeo4w/ini.bat \
	osgeo4w/nmake$BITS.opt \
	osgeo4w/package.sh \
	osgeo4w/envdiff.sed
